#!/usr/bin/env node

var AHPluginNameRegexp=/ah-(?:.+)-plugin/;

var path=require("path");
var fs=require("fs");
var spawn=require("child_process").spawn;
var pkg=require(path.join(__dirname, "../", "package.json"));


// NOTE: cwd is the npm install dir root - i.e. the dir you ran npm install tdp-ah-cms in

fs.readdir(path.join(__dirname, "../"), function(err, files)
{
	if(files.indexOf("actionhero")!==-1)
	{
		console.log("../actionhero exists, exiting");
		process.exit(0);
	}
	else
	{
		console.log("../actionhero does not exist, probably safe to continue. Let's go...");

		// Remove the scripts.postinstall parameter as it'll cause this script to be re-run when `npm install` is run (below)
		delete pkg.scripts.postinstall;

		// Add a script for the CMS installer (generated below)
		pkg.scripts["installer"]="./scripts/install-cms.sh";

		// Add a start script
		pkg.scripts.start="./node_modules/actionhero/bin/actionhero start";

		// Make npm test be recursive
		pkg.scripts.test="mocha --recursive";

		// Might want to prettify the pkg JSON here

		// Write out the updated package.json file to the root
		fs.writeFile(path.join(__dirname, "../../../package.json"), JSON.stringify(pkg), function(err, fd)
		{
			if(err)
			{
				console.log("Couldn't write output file for updated package.json: %s", err);
			}
			else
			{
				// Get the list of dependencies
				var deps=pkg.dependencies;
				var i;
				var plugins=[];

				for(i in deps)
				{
					if(i.match(AHPluginNameRegexp))
					{
						plugins.push(i);
					}
				}

				// Make a new dir in the project root to contain the installer script
				fs.mkdir(path.join(__dirname, "../../../scripts"), function(err)
				{
					if(err)
					{
						console.log("ERROR: "+err);
					}
					else
					{
						var installerScriptContent="npm install --production\r\n \
						./node_modules/actionhero/bin/actionhero generate --plugins="+plugins.toString()+" \r\n";

						// Symlink the plugin test directories in /test 
			// NOTE: this blindly assumes there is a test dir - we should check!
						var i;
						for(i in plugins)
						{
							installerScriptContent+="ln -s  ../node_modules/"+plugins[i]+"/test/ ./test/"+plugins[i]+"; ";
						}

						// Remove the default actionhero index.html (and everything else - we don't need it for our purposes and it risks confusing things)...
						// ...so that we can serve our own (from public)
						// NOTE: we don't delete public/javascript as that contains (or will do) the AH client JS (which we *do* need)
						installerScriptContent+="rm  ./public/*.html;";
						installerScriptContent+="rm -Rf ./public/css/;";
						installerScriptContent+="rm -Rf ./public/logo/;";

						var installerScriptFilename=path.join(__dirname, "../../../scripts/install-cms.sh");

						// Write out the installer script
						fs.writeFile(installerScriptFilename, installerScriptContent, function(err, fd)
						{
							if(err)
							{
								console.log("ERROR: "+err);
							}
							else
							{
								// chmod the installer script as it needs to be executable for npm to run it
								fs.chmod(installerScriptFilename, "775", function(err)
								{
									if(err)
									{
										console.log("#winning. We're done. You can now install the system via \"npm run-script install-cms\"");
									}
									else
									{
										console.log("ERROR: "+err);
									}
								});
							}
						});
					}
				});
			}
		});
	}
});